/**	\file Application.cpp
 **	\date  2008-05-26
 **	\author grymse@alhem.net
**/
/*
Copyright (C) 2008  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "Application.h"
#include <Sockets/Exception.h>


Application::Application(int argc, char *argv[]) : m_lt("std::vector")
, m_overwrite(false)
, m_prefix(true)
{
	if (argc < 2)
	{
		printf("Usage: %s [options] <.wsdl filename>\n", *argv);
		printf("    -port <portname>       Select portType\n");
		printf("    -namespace <ns>        Define a global namespace\n");
		printf("    -classname <name>      Soap Client classname\n");
		printf("    -lt <type>             Vector type (default: std::vector)\n");
		printf("    -href                  Create C++ namespace from href instead of prefix\n");
		printf("    -overwrite             Replace existing files\n");

		exit(-1);
	}
	for (int i = 1; i < argc; i++)
		if (!strcmp(argv[i], "-port") && i < argc - 1)
			m_portname = argv[++i];
		else
		if (!strcmp(argv[i], "-overwrite"))
			m_overwrite = true;
		else
		if (!strcmp(argv[i], "-namespace") && i < argc - 1)
			m_global_ns = argv[++i];
		else
		if (!strcmp(argv[i], "-classname") && i < argc - 1)
			m_classname = argv[++i];
		else
		if (!strcmp(argv[i], "-lt") && i < argc - 1)
			m_lt = argv[++i];
		else
		if (!strcmp(argv[i], "-href"))
			m_prefix = false;
		else
			m_filename = argv[i];

	if (m_classname.empty() && !m_filename.empty())
	{
		size_t pos = m_filename.find(".");
		if (pos != std::string::npos)
		{
			m_classname = m_filename.substr(0, pos);
		}
	}
}


Application::~Application()
{
}


FILE *Application::myfopen(const std::string& filename, const std::string& mode)
{
	if (!mode.empty() && mode[0] == 'w')
	{
		FILE *fil = fopen(filename.c_str(), "r");
		if (fil)
		{
			fclose(fil);
			if (!m_overwrite)
			{
				throw Exception("File already exists: " + filename);
			}
		}
		fil = fopen(filename.c_str(), mode.c_str());
		if (!fil)
		{
			throw Exception("Couldn't create file: " + filename);
		}
		return fil;
	}
	FILE *fil = fopen(filename.c_str(), mode.c_str());
	if (!fil)
	{
		throw Exception("Couldn't open file: " + filename);
	}
	return fil;
}


