#include "Iconf.h"
#include <Sockets/XmlNode.h>
#include <Sockets/XmlException.h>

namespace AJP13 {

/** http://www.alhem.net/1.0/server */
namespace net_alhem_1_0_server {

// -----------------------------------------------------------------------------
Security::Security()
{
}


// -----------------------------------------------------------------------------
Security::Security(const Xml::XmlNode& it)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Security::FromXml(const Xml::XmlNode& it)
{
	{
		Xml::XmlNode n(it, "group");
		while (n)
		{
			net_alhem_1_0_server::Security::Group_t tmp(n);
			_element_group.push_back( tmp );
			++n;
		}
	}
	if (!it.Exists("basicAuthentication"))
		throw Xml::XmlException("missing required node in element 'security': basicAuthentication");
	{
		Xml::XmlNode n(it, "basicAuthentication");
		if (n)
			_element_basicAuthentication.FromXml( n );
	}
	if (!it.Exists("authorization"))
		throw Xml::XmlException("missing required node in element 'security': authorization");
	{
		Xml::XmlNode n(it, "authorization");
		if (n)
			_element_authorization.FromXml( n );
	}
}


// -----------------------------------------------------------------------------
const std::string Security::ToString(const std::string& name) const
{
	std::string r = "<" + name + ">";
	for (std::vector<net_alhem_1_0_server::Security::Group_t>::const_iterator it = _element_group.begin(); it != _element_group.end(); it++)
	{
		const net_alhem_1_0_server::Security::Group_t& ref = *it;
		r += ref.ToString("group");
	}
	r += _element_basicAuthentication.ToString("basicAuthentication");
	r += _element_authorization.ToString("authorization");
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Security::IsSet() const
{
	return GroupIsSet() || BasicAuthenticationIsSet() || AuthorizationIsSet();
}


// -----------------------------------------------------------------------------
const std::vector<net_alhem_1_0_server::Security::Group_t>& Security::GetGroup() const
{
	return _element_group;
}


std::vector<net_alhem_1_0_server::Security::Group_t>& Security::GetGroup()
{
	return _element_group;
}


void Security::SetGroup(const std::vector<net_alhem_1_0_server::Security::Group_t>& group)
{
	_element_group = group;
}


// -----------------------------------------------------------------------------
const net_alhem_1_0_server::Security::BasicAuthentication_t& Security::GetBasicAuthentication() const
{
	return _element_basicAuthentication;
}


net_alhem_1_0_server::Security::BasicAuthentication_t& Security::GetBasicAuthentication()
{
	return _element_basicAuthentication;
}


void Security::SetBasicAuthentication(const net_alhem_1_0_server::Security::BasicAuthentication_t& basicAuthentication)
{
	_element_basicAuthentication = basicAuthentication;
}


// -----------------------------------------------------------------------------
const net_alhem_1_0_server::Security::Authorization_t& Security::GetAuthorization() const
{
	return _element_authorization;
}


net_alhem_1_0_server::Security::Authorization_t& Security::GetAuthorization()
{
	return _element_authorization;
}


void Security::SetAuthorization(const net_alhem_1_0_server::Security::Authorization_t& authorization)
{
	_element_authorization = authorization;
}


// -----------------------------------------------------------------------------
bool Security::GroupIsSet() const
{
	return !_element_group.empty();
}


// -----------------------------------------------------------------------------
bool Security::BasicAuthenticationIsSet() const
{
	return _element_basicAuthentication.IsSet();
}


// -----------------------------------------------------------------------------
bool Security::AuthorizationIsSet() const
{
	return _element_authorization.IsSet();
}


// -----------------------------------------------------------------------------
Security::Group_t::Group_t()
: _attribute_name_is_set(false)
{
}


// -----------------------------------------------------------------------------
Security::Group_t::Group_t(const Xml::XmlNode& it)
: _attribute_name_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Security::Group_t::FromXml(const Xml::XmlNode& it)
{
	AttrSetName(it.GetProperty("name"));
}


// -----------------------------------------------------------------------------
const std::string Security::Group_t::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " name='" + AttrGetName() + "'";
	r += ">";
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Security::Group_t::IsSet() const
{
	return AttrNameIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Security::Group_t::AttrGetName() const
{
	return _attribute_name;
}


void Security::Group_t::AttrSetName(const std::string& name)
{
	_attribute_name = name;
	_attribute_name_is_set = true;
}


// -----------------------------------------------------------------------------
Security::BasicAuthentication_t::BasicAuthentication_t()
: _attribute_name_is_set(false)
{
}


// -----------------------------------------------------------------------------
Security::BasicAuthentication_t::BasicAuthentication_t(const Xml::XmlNode& it)
: _attribute_name_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Security::BasicAuthentication_t::FromXml(const Xml::XmlNode& it)
{
	if (!it.Exists("user"))
		throw Xml::XmlException("missing required node in element 'basicAuthentication_t': user");
	{
		Xml::XmlNode n(it, "user");
		if (n)
			_element_user.FromXml( n );
	}
	AttrSetName(it.GetProperty("name"));
}


// -----------------------------------------------------------------------------
const std::string Security::BasicAuthentication_t::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " name='" + AttrGetName() + "'";
	r += ">";
	r += _element_user.ToString("user");
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Security::BasicAuthentication_t::IsSet() const
{
	return UserIsSet() || AttrNameIsSet();
}


// -----------------------------------------------------------------------------
const net_alhem_1_0_server::Security::BasicAuthentication_t::User_t& Security::BasicAuthentication_t::GetUser() const
{
	return _element_user;
}


net_alhem_1_0_server::Security::BasicAuthentication_t::User_t& Security::BasicAuthentication_t::GetUser()
{
	return _element_user;
}


void Security::BasicAuthentication_t::SetUser(const net_alhem_1_0_server::Security::BasicAuthentication_t::User_t& user)
{
	_element_user = user;
}


// -----------------------------------------------------------------------------
bool Security::BasicAuthentication_t::UserIsSet() const
{
	return _element_user.IsSet();
}


// -----------------------------------------------------------------------------
const std::string& Security::BasicAuthentication_t::AttrGetName() const
{
	return _attribute_name;
}


void Security::BasicAuthentication_t::AttrSetName(const std::string& name)
{
	_attribute_name = name;
	_attribute_name_is_set = true;
}


// -----------------------------------------------------------------------------
Security::BasicAuthentication_t::User_t::User_t()
: _attribute_login_is_set(false)
, _attribute_auth_is_set(false)
, _attribute_name_is_set(false)
, _attribute_groups_is_set(false)
{
}


// -----------------------------------------------------------------------------
Security::BasicAuthentication_t::User_t::User_t(const Xml::XmlNode& it)
: _attribute_login_is_set(false)
, _attribute_auth_is_set(false)
, _attribute_name_is_set(false)
, _attribute_groups_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Security::BasicAuthentication_t::User_t::FromXml(const Xml::XmlNode& it)
{
	AttrSetLogin(it.GetProperty("login"));
	AttrSetAuth(it.GetProperty("auth"));
	AttrSetName(it.GetProperty("name"));
	AttrSetGroups(it.GetProperty("groups"));
}


// -----------------------------------------------------------------------------
const std::string Security::BasicAuthentication_t::User_t::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " login='" + AttrGetLogin() + "'";
	r += " auth='" + AttrGetAuth() + "'";
	r += " name='" + AttrGetName() + "'";
	r += " groups='" + AttrGetGroups() + "'";
	r += ">";
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Security::BasicAuthentication_t::User_t::IsSet() const
{
	return AttrLoginIsSet() || AttrAuthIsSet() || AttrNameIsSet() || AttrGroupsIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Security::BasicAuthentication_t::User_t::AttrGetLogin() const
{
	return _attribute_login;
}


void Security::BasicAuthentication_t::User_t::AttrSetLogin(const std::string& login)
{
	_attribute_login = login;
	_attribute_login_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Security::BasicAuthentication_t::User_t::AttrGetAuth() const
{
	return _attribute_auth;
}


void Security::BasicAuthentication_t::User_t::AttrSetAuth(const std::string& auth)
{
	_attribute_auth = auth;
	_attribute_auth_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Security::BasicAuthentication_t::User_t::AttrGetName() const
{
	return _attribute_name;
}


void Security::BasicAuthentication_t::User_t::AttrSetName(const std::string& name)
{
	_attribute_name = name;
	_attribute_name_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Security::BasicAuthentication_t::User_t::AttrGetGroups() const
{
	return _attribute_groups;
}


void Security::BasicAuthentication_t::User_t::AttrSetGroups(const std::string& groups)
{
	_attribute_groups = groups;
	_attribute_groups_is_set = true;
}


// -----------------------------------------------------------------------------
Security::Authorization_t::Authorization_t()
: _attribute_name_is_set(false)
, _attribute_level_is_set(false)
{
}


// -----------------------------------------------------------------------------
Security::Authorization_t::Authorization_t(const Xml::XmlNode& it)
: _attribute_name_is_set(false)
, _attribute_level_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Security::Authorization_t::FromXml(const Xml::XmlNode& it)
{
	{
		Xml::XmlNode n(it, "group");
		while (n)
		{
			_element_group.push_back( n.GetContent() );
			++n;
		}
	}
	{
		Xml::XmlNode n(it, "user");
		while (n)
		{
			_element_user.push_back( n.GetContent() );
			++n;
		}
	}
	AttrSetName(it.GetProperty("name"));
	AttrSetLevel(it.GetProperty("level"));
}


// -----------------------------------------------------------------------------
const std::string Security::Authorization_t::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " name='" + AttrGetName() + "'";
	r += " level='" + AttrGetLevel() + "'";
	r += ">";
	for (std::vector<std::string>::const_iterator it = _element_group.begin(); it != _element_group.end(); it++)
	{
		r += "<group>" + *it + "</group>";
	}
	for (std::vector<std::string>::const_iterator it = _element_user.begin(); it != _element_user.end(); it++)
	{
		r += "<user>" + *it + "</user>";
	}
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Security::Authorization_t::IsSet() const
{
	return GroupIsSet() || UserIsSet() || AttrNameIsSet() || AttrLevelIsSet();
}


// -----------------------------------------------------------------------------
const std::vector<std::string>& Security::Authorization_t::GetGroup() const
{
	return _element_group;
}


std::vector<std::string>& Security::Authorization_t::GetGroup()
{
	return _element_group;
}


void Security::Authorization_t::SetGroup(const std::vector<std::string>& group)
{
	_element_group = group;
}


// -----------------------------------------------------------------------------
const std::vector<std::string>& Security::Authorization_t::GetUser() const
{
	return _element_user;
}


std::vector<std::string>& Security::Authorization_t::GetUser()
{
	return _element_user;
}


void Security::Authorization_t::SetUser(const std::vector<std::string>& user)
{
	_element_user = user;
}


// -----------------------------------------------------------------------------
bool Security::Authorization_t::GroupIsSet() const
{
	return !_element_group.empty();
}


// -----------------------------------------------------------------------------
bool Security::Authorization_t::UserIsSet() const
{
	return !_element_user.empty();
}


// -----------------------------------------------------------------------------
const std::string& Security::Authorization_t::AttrGetName() const
{
	return _attribute_name;
}


void Security::Authorization_t::AttrSetName(const std::string& name)
{
	_attribute_name = name;
	_attribute_name_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Security::Authorization_t::AttrGetLevel() const
{
	return _attribute_level;
}


void Security::Authorization_t::AttrSetLevel(const std::string& level)
{
	_attribute_level = level;
	_attribute_level_is_set = true;
}


// -----------------------------------------------------------------------------
Log::Log()
: _attribute_filename_is_set(false)
, _attribute_type_is_set(false)
, _attribute_format_is_set(false)
{
}


// -----------------------------------------------------------------------------
Log::Log(const Xml::XmlNode& it)
: _attribute_filename_is_set(false)
, _attribute_type_is_set(false)
, _attribute_format_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Log::FromXml(const Xml::XmlNode& it)
{
	AttrSetFilename(it.GetProperty("filename"));
	AttrSetType(it.GetProperty("type"));
	if (it.PropertyExists("format"))
		AttrSetFormat(it.GetProperty("format"));
}


// -----------------------------------------------------------------------------
const std::string Log::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " filename='" + AttrGetFilename() + "'";
	r += " type='" + AttrGetType() + "'";
	if (AttrFormatIsSet())
		r += " format='" + AttrGetFormat() + "'";
	r += ">";
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Log::IsSet() const
{
	return AttrFilenameIsSet() || AttrTypeIsSet() || AttrFormatIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Log::AttrGetFilename() const
{
	return _attribute_filename;
}


void Log::AttrSetFilename(const std::string& filename)
{
	_attribute_filename = filename;
	_attribute_filename_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Log::AttrGetType() const
{
	return _attribute_type;
}


void Log::AttrSetType(const std::string& type)
{
	_attribute_type = type;
	_attribute_type_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Log::AttrGetFormat() const
{
	return _attribute_format;
}


void Log::AttrSetFormat(const std::string& format)
{
	_attribute_format = format;
	_attribute_format_is_set = true;
}


// -----------------------------------------------------------------------------
Hosts::Hosts()
{
}


// -----------------------------------------------------------------------------
Hosts::Hosts(const Xml::XmlNode& it)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Hosts::FromXml(const Xml::XmlNode& it)
{
	if (!it.Exists("hostDefaults"))
		throw Xml::XmlException("missing required node in element 'hosts': hostDefaults");
	{
		Xml::XmlNode n(it, "hostDefaults");
		if (n)
			_element_hostDefaults.FromXml( n );
	}
	{
		Xml::XmlNode n(it, "host");
		while (n)
		{
			net_alhem_1_0_server::Hosts::Host_t tmp(n);
			_element_host.push_back( tmp );
			++n;
		}
	}
}


// -----------------------------------------------------------------------------
const std::string Hosts::ToString(const std::string& name) const
{
	std::string r = "<" + name + ">";
	r += _element_hostDefaults.ToString("hostDefaults");
	for (std::vector<net_alhem_1_0_server::Hosts::Host_t>::const_iterator it = _element_host.begin(); it != _element_host.end(); it++)
	{
		const net_alhem_1_0_server::Hosts::Host_t& ref = *it;
		r += ref.ToString("host");
	}
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Hosts::IsSet() const
{
	return HostDefaultsIsSet() || HostIsSet();
}


// -----------------------------------------------------------------------------
const net_alhem_1_0_server::Hosts::HostDefaults_t& Hosts::GetHostDefaults() const
{
	return _element_hostDefaults;
}


net_alhem_1_0_server::Hosts::HostDefaults_t& Hosts::GetHostDefaults()
{
	return _element_hostDefaults;
}


void Hosts::SetHostDefaults(const net_alhem_1_0_server::Hosts::HostDefaults_t& hostDefaults)
{
	_element_hostDefaults = hostDefaults;
}


// -----------------------------------------------------------------------------
const std::vector<net_alhem_1_0_server::Hosts::Host_t>& Hosts::GetHost() const
{
	return _element_host;
}


std::vector<net_alhem_1_0_server::Hosts::Host_t>& Hosts::GetHost()
{
	return _element_host;
}


void Hosts::SetHost(const std::vector<net_alhem_1_0_server::Hosts::Host_t>& host)
{
	_element_host = host;
}


// -----------------------------------------------------------------------------
bool Hosts::HostDefaultsIsSet() const
{
	return _element_hostDefaults.IsSet();
}


// -----------------------------------------------------------------------------
bool Hosts::HostIsSet() const
{
	return !_element_host.empty();
}


// -----------------------------------------------------------------------------
Hosts::HostDefaults_t::HostDefaults_t()
: _element_documents_is_set(false)
, _element_scripts_is_set(false)
, _element_logs_is_set(false)
, _element_indexDocument_is_set(false)
{
}


// -----------------------------------------------------------------------------
Hosts::HostDefaults_t::HostDefaults_t(const Xml::XmlNode& it)
: _element_documents_is_set(false)
, _element_scripts_is_set(false)
, _element_logs_is_set(false)
, _element_indexDocument_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Hosts::HostDefaults_t::FromXml(const Xml::XmlNode& it)
{
	if (it.Exists("documents"))
		SetDocuments( it["documents"] );
	if (it.Exists("scripts"))
		SetScripts( it["scripts"] );
	if (it.Exists("logs"))
		SetLogs( it["logs"] );
	if (it.Exists("indexDocument"))
		SetIndexDocument( it["indexDocument"] );
	{
		Xml::XmlNode n(it, "log");
		while (n)
		{
			net_alhem_1_0_server::Log tmp(n);
			_element_log.push_back( tmp );
			++n;
		}
	}
}


// -----------------------------------------------------------------------------
const std::string Hosts::HostDefaults_t::ToString(const std::string& name) const
{
	std::string r = "<" + name + ">";
	if (_element_documents_is_set)
		r += "<documents>" + _element_documents + "</documents>";
	if (_element_scripts_is_set)
		r += "<scripts>" + _element_scripts + "</scripts>";
	if (_element_logs_is_set)
		r += "<logs>" + _element_logs + "</logs>";
	if (_element_indexDocument_is_set)
		r += "<indexDocument>" + _element_indexDocument + "</indexDocument>";
	for (std::vector<net_alhem_1_0_server::Log>::const_iterator it = _element_log.begin(); it != _element_log.end(); it++)
	{
		const net_alhem_1_0_server::Log& ref = *it;
		r += ref.ToString("log");
	}
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Hosts::HostDefaults_t::IsSet() const
{
	return DocumentsIsSet() || ScriptsIsSet() || LogsIsSet() || IndexDocumentIsSet() || LogIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Hosts::HostDefaults_t::GetDocuments() const
{
	return _element_documents;
}


void Hosts::HostDefaults_t::SetDocuments(const std::string& documents)
{
	_element_documents = documents;
	_element_documents_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::HostDefaults_t::GetScripts() const
{
	return _element_scripts;
}


void Hosts::HostDefaults_t::SetScripts(const std::string& scripts)
{
	_element_scripts = scripts;
	_element_scripts_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::HostDefaults_t::GetLogs() const
{
	return _element_logs;
}


void Hosts::HostDefaults_t::SetLogs(const std::string& logs)
{
	_element_logs = logs;
	_element_logs_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::HostDefaults_t::GetIndexDocument() const
{
	return _element_indexDocument;
}


void Hosts::HostDefaults_t::SetIndexDocument(const std::string& indexDocument)
{
	_element_indexDocument = indexDocument;
	_element_indexDocument_is_set = true;
}


// -----------------------------------------------------------------------------
const std::vector<net_alhem_1_0_server::Log>& Hosts::HostDefaults_t::GetLog() const
{
	return _element_log;
}


std::vector<net_alhem_1_0_server::Log>& Hosts::HostDefaults_t::GetLog()
{
	return _element_log;
}


void Hosts::HostDefaults_t::SetLog(const std::vector<net_alhem_1_0_server::Log>& log)
{
	_element_log = log;
}


// -----------------------------------------------------------------------------
bool Hosts::HostDefaults_t::LogIsSet() const
{
	return !_element_log.empty();
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Host_t()
: _element_documents_is_set(false)
, _element_scripts_is_set(false)
, _element_logs_is_set(false)
, _element_indexDocument_is_set(false)
, _element_pre_is_set(false)
, _attribute_name_is_set(false)
, _attribute_rootDir_is_set(false)
{
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Host_t(const Xml::XmlNode& it)
: _element_documents_is_set(false)
, _element_scripts_is_set(false)
, _element_logs_is_set(false)
, _element_indexDocument_is_set(false)
, _element_pre_is_set(false)
, _attribute_name_is_set(false)
, _attribute_rootDir_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Hosts::Host_t::FromXml(const Xml::XmlNode& it)
{
	if (it.Exists("documents"))
		SetDocuments( it["documents"] );
	if (it.Exists("scripts"))
		SetScripts( it["scripts"] );
	if (it.Exists("logs"))
		SetLogs( it["logs"] );
	if (it.Exists("indexDocument"))
		SetIndexDocument( it["indexDocument"] );
	{
		Xml::XmlNode n(it, "log");
		while (n)
		{
			net_alhem_1_0_server::Log tmp(n);
			_element_log.push_back( tmp );
			++n;
		}
	}
	{
		Xml::XmlNode n(it, "alias");
		if (n)
			_element_alias.FromXml( n );
	}
	{
		Xml::XmlNode n(it, "ssl");
		if (n)
			_element_ssl.FromXml( n );
	}
	if (it.Exists("pre"))
		SetPre( it["pre"] );
	{
		Xml::XmlNode n(it, "mount");
		while (n)
		{
			net_alhem_1_0_server::Hosts::Host_t::Mount_t tmp(n);
			_element_mount.push_back( tmp );
			++n;
		}
	}
	{
		Xml::XmlNode n(it, "secure");
		while (n)
		{
			net_alhem_1_0_server::Hosts::Host_t::Secure_t tmp(n);
			_element_secure.push_back( tmp );
			++n;
		}
	}
	AttrSetName(it.GetProperty("name"));
	AttrSetRootDir(it.GetProperty("rootDir"));
}


// -----------------------------------------------------------------------------
const std::string Hosts::Host_t::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " name='" + AttrGetName() + "'";
	r += " rootDir='" + AttrGetRootDir() + "'";
	r += ">";
	if (_element_documents_is_set)
		r += "<documents>" + _element_documents + "</documents>";
	if (_element_scripts_is_set)
		r += "<scripts>" + _element_scripts + "</scripts>";
	if (_element_logs_is_set)
		r += "<logs>" + _element_logs + "</logs>";
	if (_element_indexDocument_is_set)
		r += "<indexDocument>" + _element_indexDocument + "</indexDocument>";
	for (std::vector<net_alhem_1_0_server::Log>::const_iterator it = _element_log.begin(); it != _element_log.end(); it++)
	{
		const net_alhem_1_0_server::Log& ref = *it;
		r += ref.ToString("log");
	}
	r += _element_alias.ToString("alias");
	r += _element_ssl.ToString("ssl");
	if (_element_pre_is_set)
		r += "<pre>" + _element_pre + "</pre>";
	for (std::vector<net_alhem_1_0_server::Hosts::Host_t::Mount_t>::const_iterator it = _element_mount.begin(); it != _element_mount.end(); it++)
	{
		const net_alhem_1_0_server::Hosts::Host_t::Mount_t& ref = *it;
		r += ref.ToString("mount");
	}
	for (std::vector<net_alhem_1_0_server::Hosts::Host_t::Secure_t>::const_iterator it = _element_secure.begin(); it != _element_secure.end(); it++)
	{
		const net_alhem_1_0_server::Hosts::Host_t::Secure_t& ref = *it;
		r += ref.ToString("secure");
	}
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::IsSet() const
{
	return DocumentsIsSet() || ScriptsIsSet() || LogsIsSet() || IndexDocumentIsSet() || LogIsSet() || AliasIsSet() || SslIsSet() || PreIsSet() || MountIsSet() || SecureIsSet() || AttrNameIsSet() || AttrRootDirIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::GetDocuments() const
{
	return _element_documents;
}


void Hosts::Host_t::SetDocuments(const std::string& documents)
{
	_element_documents = documents;
	_element_documents_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::GetScripts() const
{
	return _element_scripts;
}


void Hosts::Host_t::SetScripts(const std::string& scripts)
{
	_element_scripts = scripts;
	_element_scripts_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::GetLogs() const
{
	return _element_logs;
}


void Hosts::Host_t::SetLogs(const std::string& logs)
{
	_element_logs = logs;
	_element_logs_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::GetIndexDocument() const
{
	return _element_indexDocument;
}


void Hosts::Host_t::SetIndexDocument(const std::string& indexDocument)
{
	_element_indexDocument = indexDocument;
	_element_indexDocument_is_set = true;
}


// -----------------------------------------------------------------------------
const std::vector<net_alhem_1_0_server::Log>& Hosts::Host_t::GetLog() const
{
	return _element_log;
}


std::vector<net_alhem_1_0_server::Log>& Hosts::Host_t::GetLog()
{
	return _element_log;
}


void Hosts::Host_t::SetLog(const std::vector<net_alhem_1_0_server::Log>& log)
{
	_element_log = log;
}


// -----------------------------------------------------------------------------
const net_alhem_1_0_server::Hosts::Host_t::Alias_t& Hosts::Host_t::GetAlias() const
{
	return _element_alias;
}


net_alhem_1_0_server::Hosts::Host_t::Alias_t& Hosts::Host_t::GetAlias()
{
	return _element_alias;
}


void Hosts::Host_t::SetAlias(const net_alhem_1_0_server::Hosts::Host_t::Alias_t& alias)
{
	_element_alias = alias;
}


// -----------------------------------------------------------------------------
const net_alhem_1_0_server::Hosts::Host_t::Ssl_t& Hosts::Host_t::GetSsl() const
{
	return _element_ssl;
}


net_alhem_1_0_server::Hosts::Host_t::Ssl_t& Hosts::Host_t::GetSsl()
{
	return _element_ssl;
}


void Hosts::Host_t::SetSsl(const net_alhem_1_0_server::Hosts::Host_t::Ssl_t& ssl)
{
	_element_ssl = ssl;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::GetPre() const
{
	return _element_pre;
}


void Hosts::Host_t::SetPre(const std::string& pre)
{
	_element_pre = pre;
	_element_pre_is_set = true;
}


// -----------------------------------------------------------------------------
const std::vector<net_alhem_1_0_server::Hosts::Host_t::Mount_t>& Hosts::Host_t::GetMount() const
{
	return _element_mount;
}


std::vector<net_alhem_1_0_server::Hosts::Host_t::Mount_t>& Hosts::Host_t::GetMount()
{
	return _element_mount;
}


void Hosts::Host_t::SetMount(const std::vector<net_alhem_1_0_server::Hosts::Host_t::Mount_t>& mount)
{
	_element_mount = mount;
}


// -----------------------------------------------------------------------------
const std::vector<net_alhem_1_0_server::Hosts::Host_t::Secure_t>& Hosts::Host_t::GetSecure() const
{
	return _element_secure;
}


std::vector<net_alhem_1_0_server::Hosts::Host_t::Secure_t>& Hosts::Host_t::GetSecure()
{
	return _element_secure;
}


void Hosts::Host_t::SetSecure(const std::vector<net_alhem_1_0_server::Hosts::Host_t::Secure_t>& secure)
{
	_element_secure = secure;
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::LogIsSet() const
{
	return !_element_log.empty();
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::AliasIsSet() const
{
	return _element_alias.IsSet();
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::SslIsSet() const
{
	return _element_ssl.IsSet();
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::MountIsSet() const
{
	return !_element_mount.empty();
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::SecureIsSet() const
{
	return !_element_secure.empty();
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::AttrGetName() const
{
	return _attribute_name;
}


void Hosts::Host_t::AttrSetName(const std::string& name)
{
	_attribute_name = name;
	_attribute_name_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::AttrGetRootDir() const
{
	return _attribute_rootDir;
}


void Hosts::Host_t::AttrSetRootDir(const std::string& rootDir)
{
	_attribute_rootDir = rootDir;
	_attribute_rootDir_is_set = true;
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Alias_t::Alias_t()
: _attribute_name_is_set(false)
, _attribute_redirect_is_set(false)
{
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Alias_t::Alias_t(const Xml::XmlNode& it)
: _attribute_name_is_set(false)
, _attribute_redirect_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Hosts::Host_t::Alias_t::FromXml(const Xml::XmlNode& it)
{
	AttrSetName(it.GetProperty("name"));
	AttrSetRedirect(it.GetProperty("redirect"));
}


// -----------------------------------------------------------------------------
const std::string Hosts::Host_t::Alias_t::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " name='" + AttrGetName() + "'";
	r += " redirect='" + AttrGetRedirect() + "'";
	r += ">";
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::Alias_t::IsSet() const
{
	return AttrNameIsSet() || AttrRedirectIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Alias_t::AttrGetName() const
{
	return _attribute_name;
}


void Hosts::Host_t::Alias_t::AttrSetName(const std::string& name)
{
	_attribute_name = name;
	_attribute_name_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Alias_t::AttrGetRedirect() const
{
	return _attribute_redirect;
}


void Hosts::Host_t::Alias_t::AttrSetRedirect(const std::string& redirect)
{
	_attribute_redirect = redirect;
	_attribute_redirect_is_set = true;
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Ssl_t::Ssl_t()
: _element_certFile_is_set(false)
, _element_keyFile_is_set(false)
, _element_keyAuth_is_set(false)
{
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Ssl_t::Ssl_t(const Xml::XmlNode& it)
: _element_certFile_is_set(false)
, _element_keyFile_is_set(false)
, _element_keyAuth_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Hosts::Host_t::Ssl_t::FromXml(const Xml::XmlNode& it)
{
	if (!it.Exists("certFile"))
		throw Xml::XmlException("missing required node in element 'ssl_t': certFile");
	SetCertFile( it["certFile"] );
	if (!it.Exists("keyFile"))
		throw Xml::XmlException("missing required node in element 'ssl_t': keyFile");
	SetKeyFile( it["keyFile"] );
	if (!it.Exists("keyAuth"))
		throw Xml::XmlException("missing required node in element 'ssl_t': keyAuth");
	SetKeyAuth( it["keyAuth"] );
}


// -----------------------------------------------------------------------------
const std::string Hosts::Host_t::Ssl_t::ToString(const std::string& name) const
{
	std::string r = "<" + name + ">";
	r += "<certFile>" + _element_certFile + "</certFile>";
	r += "<keyFile>" + _element_keyFile + "</keyFile>";
	r += "<keyAuth>" + _element_keyAuth + "</keyAuth>";
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::Ssl_t::IsSet() const
{
	return CertFileIsSet() || KeyFileIsSet() || KeyAuthIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Ssl_t::GetCertFile() const
{
	return _element_certFile;
}


void Hosts::Host_t::Ssl_t::SetCertFile(const std::string& certFile)
{
	_element_certFile = certFile;
	_element_certFile_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Ssl_t::GetKeyFile() const
{
	return _element_keyFile;
}


void Hosts::Host_t::Ssl_t::SetKeyFile(const std::string& keyFile)
{
	_element_keyFile = keyFile;
	_element_keyFile_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Ssl_t::GetKeyAuth() const
{
	return _element_keyAuth;
}


void Hosts::Host_t::Ssl_t::SetKeyAuth(const std::string& keyAuth)
{
	_element_keyAuth = keyAuth;
	_element_keyAuth_is_set = true;
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Mount_t::Mount_t()
: _attribute_uri_is_set(false)
, _attribute_jobber_is_set(false)
, _attribute_strip_is_set(false)
, _attribute_path_is_set(false)
{
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Mount_t::Mount_t(const Xml::XmlNode& it)
: _attribute_uri_is_set(false)
, _attribute_jobber_is_set(false)
, _attribute_strip_is_set(false)
, _attribute_path_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Hosts::Host_t::Mount_t::FromXml(const Xml::XmlNode& it)
{
	AttrSetUri(it.GetProperty("uri"));
	AttrSetJobber(it.GetProperty("jobber"));
	if (it.PropertyExists("strip"))
		AttrSetStrip(it.GetProperty("strip"));
	if (it.PropertyExists("path"))
		AttrSetPath(it.GetProperty("path"));
}


// -----------------------------------------------------------------------------
const std::string Hosts::Host_t::Mount_t::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " uri='" + AttrGetUri() + "'";
	r += " jobber='" + AttrGetJobber() + "'";
	if (AttrStripIsSet())
		r += " strip='" + AttrGetStrip() + "'";
	if (AttrPathIsSet())
		r += " path='" + AttrGetPath() + "'";
	r += ">";
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::Mount_t::IsSet() const
{
	return AttrUriIsSet() || AttrJobberIsSet() || AttrStripIsSet() || AttrPathIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Mount_t::AttrGetUri() const
{
	return _attribute_uri;
}


void Hosts::Host_t::Mount_t::AttrSetUri(const std::string& uri)
{
	_attribute_uri = uri;
	_attribute_uri_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Mount_t::AttrGetJobber() const
{
	return _attribute_jobber;
}


void Hosts::Host_t::Mount_t::AttrSetJobber(const std::string& jobber)
{
	_attribute_jobber = jobber;
	_attribute_jobber_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Mount_t::AttrGetStrip() const
{
	return _attribute_strip;
}


void Hosts::Host_t::Mount_t::AttrSetStrip(const std::string& strip)
{
	_attribute_strip = strip;
	_attribute_strip_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Mount_t::AttrGetPath() const
{
	return _attribute_path;
}


void Hosts::Host_t::Mount_t::AttrSetPath(const std::string& path)
{
	_attribute_path = path;
	_attribute_path_is_set = true;
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Secure_t::Secure_t()
: _attribute_uri_is_set(false)
, _attribute_basicAuthentication_is_set(false)
, _attribute_authorization_is_set(false)
{
}


// -----------------------------------------------------------------------------
Hosts::Host_t::Secure_t::Secure_t(const Xml::XmlNode& it)
: _attribute_uri_is_set(false)
, _attribute_basicAuthentication_is_set(false)
, _attribute_authorization_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Hosts::Host_t::Secure_t::FromXml(const Xml::XmlNode& it)
{
	AttrSetUri(it.GetProperty("uri"));
	if (it.PropertyExists("basicAuthentication"))
		AttrSetBasicAuthentication(it.GetProperty("basicAuthentication"));
	if (it.PropertyExists("authorization"))
		AttrSetAuthorization(it.GetProperty("authorization"));
}


// -----------------------------------------------------------------------------
const std::string Hosts::Host_t::Secure_t::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " uri='" + AttrGetUri() + "'";
	if (AttrBasicAuthenticationIsSet())
		r += " basicAuthentication='" + AttrGetBasicAuthentication() + "'";
	if (AttrAuthorizationIsSet())
		r += " authorization='" + AttrGetAuthorization() + "'";
	r += ">";
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Hosts::Host_t::Secure_t::IsSet() const
{
	return AttrUriIsSet() || AttrBasicAuthenticationIsSet() || AttrAuthorizationIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Secure_t::AttrGetUri() const
{
	return _attribute_uri;
}


void Hosts::Host_t::Secure_t::AttrSetUri(const std::string& uri)
{
	_attribute_uri = uri;
	_attribute_uri_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Secure_t::AttrGetBasicAuthentication() const
{
	return _attribute_basicAuthentication;
}


void Hosts::Host_t::Secure_t::AttrSetBasicAuthentication(const std::string& basicAuthentication)
{
	_attribute_basicAuthentication = basicAuthentication;
	_attribute_basicAuthentication_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Hosts::Host_t::Secure_t::AttrGetAuthorization() const
{
	return _attribute_authorization;
}


void Hosts::Host_t::Secure_t::AttrSetAuthorization(const std::string& authorization)
{
	_attribute_authorization = authorization;
	_attribute_authorization_is_set = true;
}


// -----------------------------------------------------------------------------
Network::Network()
{
}


// -----------------------------------------------------------------------------
Network::Network(const Xml::XmlNode& it)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Network::FromXml(const Xml::XmlNode& it)
{
	if (!it.Exists("bind"))
		throw Xml::XmlException("missing required node in element 'network': bind");
	{
		Xml::XmlNode n(it, "bind");
		while (n)
		{
			net_alhem_1_0_server::Network::Bind_t tmp(n);
			_element_bind.push_back( tmp );
			++n;
		}
	}
}


// -----------------------------------------------------------------------------
const std::string Network::ToString(const std::string& name) const
{
	std::string r = "<" + name + ">";
	for (std::vector<net_alhem_1_0_server::Network::Bind_t>::const_iterator it = _element_bind.begin(); it != _element_bind.end(); it++)
	{
		const net_alhem_1_0_server::Network::Bind_t& ref = *it;
		r += ref.ToString("bind");
	}
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Network::IsSet() const
{
	return BindIsSet();
}


// -----------------------------------------------------------------------------
const std::vector<net_alhem_1_0_server::Network::Bind_t>& Network::GetBind() const
{
	return _element_bind;
}


std::vector<net_alhem_1_0_server::Network::Bind_t>& Network::GetBind()
{
	return _element_bind;
}


void Network::SetBind(const std::vector<net_alhem_1_0_server::Network::Bind_t>& bind)
{
	_element_bind = bind;
}


// -----------------------------------------------------------------------------
bool Network::BindIsSet() const
{
	return !_element_bind.empty();
}


// -----------------------------------------------------------------------------
Network::Bind_t::Bind_t()
: _attribute_port(0)
, _attribute_interface_is_set(false)
, _attribute_port_is_set(false)
, _attribute_protocol_is_set(false)
, _attribute_host_is_set(false)
{
}


// -----------------------------------------------------------------------------
Network::Bind_t::Bind_t(const Xml::XmlNode& it)
: _attribute_port(0)
, _attribute_interface_is_set(false)
, _attribute_port_is_set(false)
, _attribute_protocol_is_set(false)
, _attribute_host_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Network::Bind_t::FromXml(const Xml::XmlNode& it)
{
	AttrSetInterface(it.GetProperty("interface"));
	AttrSetPort( atoi(it.GetProperty("port").c_str()) );
	AttrSetProtocol(it.GetProperty("protocol"));
	if (it.PropertyExists("host"))
		AttrSetHost(it.GetProperty("host"));
}


// -----------------------------------------------------------------------------
const std::string Network::Bind_t::ToString(const std::string& name) const
{
	std::string r = "<" + name;
	r += " interface='" + AttrGetInterface() + "'";
	{
		char slask[100];
		sprintf(slask, "%d", AttrGetPort());
		r = r + " port='" + slask + "'";
	}
	r += " protocol='" + AttrGetProtocol() + "'";
	if (AttrHostIsSet())
		r += " host='" + AttrGetHost() + "'";
	r += ">";
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Network::Bind_t::IsSet() const
{
	return AttrInterfaceIsSet() || AttrPortIsSet() || AttrProtocolIsSet() || AttrHostIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Network::Bind_t::AttrGetInterface() const
{
	return _attribute_interface;
}


void Network::Bind_t::AttrSetInterface(const std::string& interface)
{
	_attribute_interface = interface;
	_attribute_interface_is_set = true;
}


// -----------------------------------------------------------------------------
int Network::Bind_t::AttrGetPort() const
{
	return _attribute_port;
}


void Network::Bind_t::AttrSetPort(int port)
{
	_attribute_port = port;
	_attribute_port_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Network::Bind_t::AttrGetProtocol() const
{
	return _attribute_protocol;
}


void Network::Bind_t::AttrSetProtocol(const std::string& protocol)
{
	_attribute_protocol = protocol;
	_attribute_protocol_is_set = true;
}


// -----------------------------------------------------------------------------
const std::string& Network::Bind_t::AttrGetHost() const
{
	return _attribute_host;
}


void Network::Bind_t::AttrSetHost(const std::string& host)
{
	_attribute_host = host;
	_attribute_host_is_set = true;
}


// -----------------------------------------------------------------------------
Configuration_t::Configuration_t()
: _element_rootDir_is_set(false)
{
}


// -----------------------------------------------------------------------------
Configuration_t::Configuration_t(const Xml::XmlNode& it)
: _element_rootDir_is_set(false)
{
	FromXml(it);
}


// -----------------------------------------------------------------------------
void Configuration_t::FromXml(const Xml::XmlNode& it)
{
	if (!it.Exists("rootDir"))
		throw Xml::XmlException("missing required node in element 'configuration_t': rootDir");
	SetRootDir( it["rootDir"] );
	if (!it.Exists("security"))
		throw Xml::XmlException("missing required node in element 'configuration_t': security");
	{
		Xml::XmlNode n(it, "security");
		if (n)
			_element_security.FromXml( n );
	}
	if (!it.Exists("hosts"))
		throw Xml::XmlException("missing required node in element 'configuration_t': hosts");
	{
		Xml::XmlNode n(it, "hosts");
		if (n)
			_element_hosts.FromXml( n );
	}
	if (!it.Exists("network"))
		throw Xml::XmlException("missing required node in element 'configuration_t': network");
	{
		Xml::XmlNode n(it, "network");
		if (n)
			_element_network.FromXml( n );
	}
}


// -----------------------------------------------------------------------------
const std::string Configuration_t::ToString(const std::string& name) const
{
	std::string r = "<" + name + ">";
	r += "<rootDir>" + _element_rootDir + "</rootDir>";
	r += _element_security.ToString("security");
	r += _element_hosts.ToString("hosts");
	r += _element_network.ToString("network");
	r += "</" + name + ">";
	return r;
}


// -----------------------------------------------------------------------------
bool Configuration_t::IsSet() const
{
	return RootDirIsSet() || SecurityIsSet() || HostsIsSet() || NetworkIsSet();
}


// -----------------------------------------------------------------------------
const std::string& Configuration_t::GetRootDir() const
{
	return _element_rootDir;
}


void Configuration_t::SetRootDir(const std::string& rootDir)
{
	_element_rootDir = rootDir;
	_element_rootDir_is_set = true;
}


// -----------------------------------------------------------------------------
const net_alhem_1_0_server::Security& Configuration_t::GetSecurity() const
{
	return _element_security;
}


net_alhem_1_0_server::Security& Configuration_t::GetSecurity()
{
	return _element_security;
}


void Configuration_t::SetSecurity(const net_alhem_1_0_server::Security& security)
{
	_element_security = security;
}


// -----------------------------------------------------------------------------
const net_alhem_1_0_server::Hosts& Configuration_t::GetHosts() const
{
	return _element_hosts;
}


net_alhem_1_0_server::Hosts& Configuration_t::GetHosts()
{
	return _element_hosts;
}


void Configuration_t::SetHosts(const net_alhem_1_0_server::Hosts& hosts)
{
	_element_hosts = hosts;
}


// -----------------------------------------------------------------------------
const net_alhem_1_0_server::Network& Configuration_t::GetNetwork() const
{
	return _element_network;
}


net_alhem_1_0_server::Network& Configuration_t::GetNetwork()
{
	return _element_network;
}


void Configuration_t::SetNetwork(const net_alhem_1_0_server::Network& network)
{
	_element_network = network;
}


// -----------------------------------------------------------------------------
bool Configuration_t::SecurityIsSet() const
{
	return _element_security.IsSet();
}


// -----------------------------------------------------------------------------
bool Configuration_t::HostsIsSet() const
{
	return _element_hosts.IsSet();
}


// -----------------------------------------------------------------------------
bool Configuration_t::NetworkIsSet() const
{
	return _element_network.IsSet();
}


} // namespace net_alhem_1_0_server

} // namespace AJP13

