/**	\file wsdl.h
 **	\date  2008-05-26
 **	\author grymse@alhem.net
**/
/*
Copyright (C) 2008  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _Wsdl_H
#define _Wsdl_H

#include <string>
#include <list>
#include <Sockets/XmlNode.h>


class Application;

class Wsdl
{
	class NSFil {
	public:
		NSFil(FILE *fil, const std::string& ns, const std::string& href = "") : m_fil(fil), m_ns(ns) {
			if (!m_ns.empty())
			{
				if (!href.empty())
					fprintf(m_fil, "/** %s */\n", href.c_str());
				fprintf(m_fil, "namespace %s {\n\n", m_ns.c_str());
			}
		}
		~NSFil() {
			if (!m_ns.empty())
			{
				fprintf(m_fil, "} // namespace %s\n\n", m_ns.c_str());
			}
		}
	private:
		FILE *m_fil;
		std::string m_ns;
	};

public:
	Wsdl(Application& appl, Xml::XmlDocument& doc);
	~Wsdl();

	Application& GetApplication() const { return m_appl; }

	const std::string GetTargetNamespace() const;
	const std::string GetTargetNamespacePrefix() const;

	const std::string GetNamespacePrefix(const std::string& href) const;

	std::list<std::string> GetPorts() const;
	const std::string GetPortname() const;
	std::list<std::string> GetOperations(const std::string& portname) const;

	const std::string GetSoapAction(const std::string& portname, const std::string& op) const;

	const std::string GetMessage(const std::string& portname, const std::string& op, const std::string& type) const;

	void ShowMessage(const std::string& msgname);
	void ShowElement(const Xml::XmlNode& element_node);
	void ShowComplexType(const Xml::XmlNode& complex);

	std::list<std::pair<std::string, Xml::XmlNode> > GetMessageParameters(const std::string& msgname, std::string& path) const;

	Xml::XmlNode GetPortTypeOp(const std::string& portname, const std::string& op) const;
	Xml::XmlNode GetBindingOp(const std::string& portname, const std::string& op) const;
	Xml::XmlNode GetMessage(const std::string& msgname) const;
	Xml::XmlNode GetSchema(const std::string& target_ns) const;
	Xml::XmlNode GetElement(const std::string& name) const;
	Xml::XmlNode FindComplexType(const std::string& type) const;
	Xml::XmlNode FindSimpleType(const std::string& type) const;
	Xml::XmlNode GetElement(const Xml::XmlNode& node, const std::string& name) const;

//	Xml::XmlNode FindComplexType(const Xml::XmlNode& node, const std::string& type) const;

	void CreateInterface();
	void CreateImplementation();

	const std::string& XmlSchemaPrefix() const { return m_xml_schema_prefix; }

	const std::string GetParent(const std::string& classname) const;

	const std::list<std::string>& ComplexTypes(const std::string& ns_prefix) const;
	const std::list<std::string>& SimpleTypes(const std::string& ns_prefix) const;

	const std::string Wsdl::NamespaceFromHref(const std::string& href) const;
	const std::string Wsdl::NamespaceFromPrefix(const std::string& prefix) const;

	/** Return prefix or mangled href */
	const std::string GetNamespace(const std::string& prefix) const;
	const std::string GetRealPrefix(const std::string& prefix) const;
	const std::string GetRealType(const std::string& type) const;

private:
	void ParseElements();
	void ParseElements(const Xml::XmlNode& node, const std::string& ns_prefix, const std::string& parent_class = "");
	void ParseTypes(const Xml::XmlNode& node, const std::string& ns_prefix);
	void ParseComplexType(const Xml::XmlNode& node, const std::string& ns_prefix, const std::string& name);

	Application& m_appl;
	Xml::XmlDocument& m_doc;
	/** Map element name to typename */
	std::map<std::string, /* namespace prefix */
		std::map<std::string, /* element name */
			std::string> > m_element; // typename
	/** List of complexType names in each namespace prefix */
	std::map<std::string, std::list<std::string> > m_complexType;
	/** List of simpleType names in each namespace prefix */
	std::map<std::string, std::list<std::string> > m_simpleType;
	/** Map complexType names in each namespace prefix to Xml::XmlNode of definition */
	std::map<std::string, std::map<std::string, Xml::XmlNode> > m_complexTypeMap;
	/** Map simpleType names in each namespace prefix to Xml::XmlNode of definition */
	std::map<std::string, std::map<std::string, Xml::XmlNode> > m_simpleTypeMap;

	/** parent class map */
	std::map<std::string, std::string> m_parent;

	std::string m_xml_schema_prefix;
};




#endif // _Wsdl_H
